Introduction
===

You can monitor your Node.js applications in production with the AppDynamics Node.js Agent. The Node.js Agent enables real-time visibility of server health, server performance, and resource usage.

Performance issues can result in a slow or unavailable website. The Node.js Agent uses process-level visibility and proactive monitoring to reveal the internal workings of your application and infrastructure with policies, troubleshooting, and performance/capacity optimization. You can use process snapshots to examine your code execution at the multi-process level and identify the root cause of slow performance in minutes.

For supported Node.js versions, operating systems, and backends, see [Node.js Supported Environments](https://docs.appdynamics.com/display/PRO21/Node.js+Supported+Environments).
   
   
     
Before You Begin
--
To start the Node.js Agent, complete the following:

- Obtain an AppDynamics account with a running Controller. 
- To sign up for a 15-day free trial, see [AppDynamics Free Trial](https://www.appdynamics.com/free-trial/). You should receive an email with your Controller host and port.
      
      
Install the Node.js Agent
--
AppDynamics recommends that you update the Node.js Agent with every SaaS Controller release. To ensure that your agent contains the latest functionality and fixes, regularly update to a major Node.js Agent version.

All Node.js Agent versions are compatible with major Controller versions 4.4.1.0 and later. For certain features with Controller version dependencies, you will need to upgrade your Controller. 

For more information, see [Agent and Controller Compatibility](https://docs.appdynamics.com/display/PRO21/Agent+and+Controller+Compatibility).

To install the Node.js Agent, insert the following command in the root directory of your instrumented Node.js application:

```sh
npm install appdynamics
```

If you know you want the latest published agent version, you can use the `next` tag:
```sh
npm install appdynamics@next
```

If you know which specific version of the Node.js agent you want to install, you can specify it:

```sh
npm install appdynamics@<version>
```

Paste the following in your application as the very first line of your application source code, before any other require statement:  

```sh
require('appdynamics').profile({
  controllerHostName: '<controller host name>',
  controllerPort: <controller port number>,
  controllerSslEnabled: false,  // Set to true if controllerPort is SSL
  accountName: '<AppDynamics_account_name>',
  accountAccessKey: '<AppDynamics_account_key>',
  applicationName: 'your_app_name',
  tierName: 'choose_a_tier_name',
  nodeName: 'choose_a_node_name'
 });
```

For more information, see [Install the Node.js Agent](https://docs.appdynamics.com/display/PRO21/Install+the+Node.js+Agent) and [Application Monitoring](https://docs.appdynamics.com/display/PRO21/Application+Monitoring).

<br />
<br />


--

<br />
<br />
<br />

Copyright (c) AppDynamics, Inc. 2021 CA. All rights reserved.

